#!/usr/bin/env python
# ldap_bind.py 
#
# CONFIDENTIAL - SOURCE MATERIALS
#
# The code is a part of ProtoVer testsuite - use only under the license agreement
# specified in LICENSE.txt in your ProtoVer distribution
#
# Copyright 2005-2006, GLEG Ltd.

import sys
import string
import time
import threading

sys.path.append("libs")
sys.path.append("../libs")
sys.path.append("../../libs")

from utils import *

import ldaprun

DESCRIPTION="LDAPBindRequest"
VERSION="1.0"
HELP="""
The Bind Request is defined as follows:

        BindRequest ::= [APPLICATION 0] SEQUENCE {
                version                 INTEGER (1 .. 127),
                name                    LDAPDN,
                authentication          AuthenticationChoice }

        AuthenticationChoice ::= CHOICE {
                simple                  [0] OCTET STRING,
                                         -- 1 and 2 reserved
                sasl                    [3] SaslCredentials }

        SaslCredentials ::= SEQUENCE {
                mechanism               LDAPString,
                credentials             OCTET STRING OPTIONAL }

   Parameters of the Bind Request are:

   - version: A version number indicating the version of the protocol to
     be used in this protocol session.  This document describes version
     3 of the LDAP protocol.  Note that there is no version negotiation,
     and the client just sets this parameter to the version it desires.

   - name: The name of the directory object that the client wishes to
     bind as.  This field may take on a null value (a zero length
     string) for the purposes of anonymous binds, when authentication
     has been performed at a lower layer, or when using SASL credentials
     with a mechanism that includes the LDAPDN in the credentials.

   - authentication: information used to authenticate the name, if any,
     provided in the Bind Request.

"""
ASN1HELP="""
    0:d=0  hl=2 l=  14 cons: SEQUENCE
    2:d=1  hl=2 l=   1 prim:  INTEGER           :01
    5:d=1  hl=2 l=   9 cons:  appl [ 0 ]
    7:d=2  hl=2 l=   1 prim:   INTEGER           :03
   10:d=2  hl=2 l=   2 prim:   OCTET STRING      :DN
   14:d=2  hl=2 l=   0 prim:   cont [ 0 ]
"""

OPTIONS=ldaprun.OPTIONS

class Runner(ldaprun.LDAPRunner):
        def __init__(self, event, opts, logger):
                ldaprun.LDAPRunner.__init__(self, event, opts, logger, DESCRIPTION, VERSION)
		self.berID = 0

	def generate(self,f):
		self.berID += 1
		return "\x30\x0e\x02\x01\x01\x60\x09\x02\x01\x03\x04\x02\x44\x4e\x80\x00"

if __name__ == "__main__":
	print "%s groups of tests version %s" %(DESCRIPTION, VERSION)
	opts = ldaprun.LDAPRunner.getopts(HELP)
        event = threading.Event()
        event.set()
        r = Runner(event, opts, TextLogger())
        r.run()

