#!/usr/bin/env python
# serverinterface.py
#
# CONFIDENTIAL - SOURCE MATERIALS
#
# The code is a part of ProtoVer testsuite - use only under the license agreement
# specified in LICENSE.txt in your ProtoVer distribution
#
# Copyright 2005-2006, GLEG Ltd.


import sys
import struct
import time
import BaseHTTPServer
import getopt
import urlparse
import cgi
import urllib
import time
import threading

sys.path.append("libs")
sys.path.append("../libs")
sys.path.append("../../libs")

import utils

VERSION="1.4"
PROTOCOL="LDAP"
DESCRIPTION="ProtoVer Server interface"

TEMPLATES="./templates"

alltestmodules = []
alltestgroups = []
allhandlers = []
allsessions = []

ASN1HELP_NOTE="""
Each line starts with the offset in decimal. 
d=XX specifies the current depth. 
The depth is increased within the scope of any SET or SEQUENCE. 
hl=XX gives the header length (tag and length octets) of the current type. 
l=XX gives the length of the contents octets.
"""

class BufferedLogger:
        def __init__(self):
		self.buffer=""
	def clear(self):
		self.buffer=""
        def write(self, text):
		# clear buffer if there is too much data
		if len(self.buffer) > 65000:
			self.buffer = "[truncated]<br><br>"
                self.buffer += text

class BaseHandler:
	def __init__(self):
		self.args = {}
		self.data = ""

	def set_arg(self, name, val):
		self.args[name] = val
	
class IndexHandler(BaseHandler):
	def __init__(self):
		BaseHandler.__init__(self)

	def test(self, path):
		if path == "/":
			return 1
		if path == "/tree":
			return 1
		return 0
	
	def handle_index(self):
		fp = open("%s/index.html"% TEMPLATES, "rb")
		data = fp.read()
		fp.close()	

		data = data.replace("<PROTOCOL>", PROTOCOL)
		data = data.replace("<VERSION>",VERSION)	

		self.data = data

	def handle_tree(self):
		s = ""
		i = 0
		for g in alltestgroups:
			s += "<tr>"
			s += "<td>&nbsp;&nbsp</td>"	
			s+= "<td>"
			s += "<a href=/module?action=open&module_id=%d target=module>%s</a>" % (i,g)
			s += "</td></tr>\r\n"
			i += 1
	
		fp=open("%s/tree.html" % TEMPLATES, "rb")
		data = fp.read()
		fp.close()

		data = data.replace("<TABLE>",s)

		self.data = data

	def run(self):
		path = self.args["path"]
		if path == "/":
			self.handle_index()
		
		if path == "/tree":
			self.handle_tree()

		return self.data

class ModuleHandler(BaseHandler):
	def __init__(self):
		BaseHandler.__init__(self)
		self.action="open"	
		self.module_id=0
	
	def test(self, path):
		if path == "/module":
			return 1

		return 0

	def handle_help(self):
		fp = open("%s/module_help.html"%TEMPLATES,"rb")
		data =fp.read()
		fp.close()
		   
		mod = None
                if self.module_id >= 0 and self.module_id < len(alltestmodules):
                        mod = alltestmodules[self.module_id]
                else:
                        mod = alltestmodules[0]
                        self.module_id = 0

		help = getattr(mod, "HELP")
                data = data.replace("<MODULE_ID>", str(self.module_id))
                data = data.replace("<HELP>", help)
                self.data=data

	
        def handle_asn1help(self):
                fp = open("%s/module_help.html"%TEMPLATES,"rb")
                data =fp.read()
                fp.close()

                mod = None
                if self.module_id >= 0 and self.module_id < len(alltestmodules):
                        mod = alltestmodules[self.module_id]
                else:
                        mod = alltestmodules[0]
                        self.module_id = 0
   
		help = ASN1HELP_NOTE
                help += getattr(mod, "ASN1HELP")
                data = data.replace("<MODULE_ID>", str(self.module_id))
                data = data.replace("<HELP>", help)
                self.data=data

	

	def handle_open(self):
		fp = open("%s/module_open.html"%TEMPLATES,"rb")
		data = fp.read()
		fp.close()

		mod = None
		if self.module_id >= 0 and self.module_id < len(alltestmodules):
			mod = alltestmodules[self.module_id]
		else:
			mod = alltestmodules[0]	
			self.module_id = 0
	
		descr =getattr(mod, "DESCRIPTION")	
	
		data = data.replace("<MODULE_ID>", str(self.module_id))
		data = data.replace("<HELP>", descr)
		self.data=data

	def handle_run(self):
		params = self.args["params"]

		module_id = int(params["module_id"][0])
		if module_id < 0 or module_id >= len(alltestmodules):
			module_id = 0	

		# Start the module and add new session
		opts = {}
		opts["host"] = params["host"][0]
		opts["port"] = int(params["port"][0])
		opts["timeout"] = float(params["timeout"][0])
		opts["start"] = int(params["start"][0])	
		opts["num"] = int(params["num"][0])
		opts["read"] = int(params["read"][0])
		opts["delay"] = float(params["delay"][0])
		opts["debug"] = int(params["debug"][0])	
	
		mod = alltestmodules[module_id]
                logger = BufferedLogger()
                runner = getattr(mod, "Runner")(threading.Event(), opts, logger)
                runner.setDaemon(1)
                runner.event.set()
                runner.start()

		sess = {}
		sess["name"] = alltestgroups[module_id]
		sess["starttime"]= time.ctime(time.time())
		sess["host"] = opts["host"]
		sess["port"] = opts["port"]
		sess["runner"] = runner
		sess["status"] = "running"

		allsessions.append(sess)

		print "Running %s against %s:%d" %(sess["name"],sess["host"],sess["port"])

		fp=open("%s/module_success.html"%TEMPLATES,"rb")
		data = fp.read()
		fp.close()

		data = data.replace("<NAME>",sess["name"])
		data = data.replace("<HOST>",sess["host"])
		data = data.replace("<PORT>",str(sess["port"]))
		data = data.replace("<MODULE_ID>", str(module_id))

		self.data = data

	def run(self):
		params = self.args["params"]

		if params.has_key("action"):
			self.action = params["action"][0]
		if params.has_key("module_id"):
			self.module_id = int(params["module_id"][0])

		if self.action =="open":
			self.handle_open()

		if self.action =="run":
			self.handle_run()

		if self.action =="help":
			self.handle_help()
		if self.action == "asn1help":
			self.handle_asn1help()

		return self.data


class SessionHandler(BaseHandler):
	def __init__(self):
		BaseHandler.__init__(self)

	def test(self, path):
		if path == "/session":
			return 1
		return 0

	def handle_display(self):
		fp = open("%s/session.html"% TEMPLATES, "rb")
		data = fp.read()
		fp.close()

		b = ""
		for i in range(0,len(allsessions)):
			sess = allsessions[i]
			runner = sess["runner"]
	
			b+="<tr>"
			b+="<td><input type=checkbox name=session_id value=%d></td>" % i
			b+="<td><nobr><a href=/log?session_id=%d target=log>" % i
			b+="[%s] %s %s %s:%d</a></nobr></td>" % (runner.status,sess["name"],
								sess["starttime"], 
								sess["host"],
								sess["port"])
			b+="</tr>"	

		data = data.replace("<SESSIONS>",b)

		self.data = data

	def handle_stop(self):
		params = self.args["params"]
		for session_id in [int(i) for i in params["session_id"]]:
			if session_id < 0 or session_id >= len(allsessions):
				continue
			runner = allsessions[session_id]["runner"]
			if runner.status.find("running") >= 0:
				runner.status = "stopped"		
				runner.event.clear()

	def handle_continue(self):
		params = self.args["params"]
                for session_id in [int(i) for i in params["session_id"]]:
			if session_id < 0 or session_id >= len(allsessions):
                                continue
                        runner = allsessions[session_id]["runner"]
			if runner.status.finished("stopped") >= 0:
                        	runner.event.set()
                        	runner.status = "running (continue again)"

	def handle_cancel(self):
		params = self.args["params"]
                for session_id in [int(i) for i in params["session_id"]]:
                        if session_id < 0 or session_id >= len(allsessions):
                                continue
                        runner = allsessions[session_id]["runner"]
                        if runner.status.find("finished") == -1:
				runner.stop = 1
                        	runner.status = "finished (canceled)"


	def run(self):
		params = self.args["params"]
		
		if params.has_key("session_id"):
			if params.has_key("Stop"):
				self.handle_stop()
			elif params.has_key("Continue"):
				self.handle_continue()
			elif params.has_key("Cancel"):
				self.handle_cancel()
		
		self.handle_display()

		return self.data

class LogHandler(BaseHandler):
	def __init__(self):
		BaseHandler.__init__(self)
		self.session_id = -1	

	def test(self,path):
		if path=="/log":
			return 1
		return 0
     
	def hexdump(self, s=""):
                i = 0
                tmp = ""
                for c in s:
                        i += 1
                        tmp += " %02x" % (ord(c),)
                        if (i%16)==0: tmp += "\n"
                return tmp


	def handle_hexdump(self):
		params = self.args["params"]
		session_id = int(params["session_id"][0])		
		if session_id >=0 and session_id < len(allsessions):
			runner = allsessions[session_id]["runner"]
			packet = self.hexdump(runner.last_packet)
			fp = open("%s/log_dump.html"%TEMPLATES,"rb")
			self.data = fp.read()
			fp.close()
			self.data = self.data.replace("<DATA>", packet)
			self.data = self.data.replace("<SESSION_ID>",str(session_id))

	def handle_asciidump(self):
		params = self.args["params"]
                session_id = int(params["session_id"][0])
                if session_id >=0 and session_id < len(allsessions):
                        runner = allsessions[session_id]["runner"]
                        packet = runner.last_packet
			fp = open("%s/log_dump.html"%TEMPLATES,"rb")
                        self.data = fp.read()
			fp.close()
                        self.data = self.data.replace("<DATA>", packet)
			self.data = self.data.replace("<SESSION_ID>",str(session_id))

	def run(self):
		params = self.args["params"]

		if params.has_key("action") and params.has_key("session_id"):
			action = params["action"][0]
			if action == "hexdump":
				self.handle_hexdump()
			if action == "asciidump":
				self.handle_asciidump()
			return self.data

		fp = open("%s/log.html"%TEMPLATES,"rb")
		self.data = fp.read()
		fp.close()

		if len(allsessions) < 1:
			self.data = self.data.replace("<DATA>","")
			return self.data

		if params.has_key("session_id"):
			self.session_id = int(params["session_id"][0])
		else:
			self.session_id = len(allsessions) - 1
	
		logger = allsessions[self.session_id]["runner"].logger
		buffer = logger.buffer.replace("\n","<br>")
		self.data = self.data.replace("<DATA>",buffer)
		self.data = self.data.replace("<SESSION_ID>", str(self.session_id))
		return self.data
		

class MainRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
	def __init__(self, request, client_address, server):
		try:
			BaseHTTPServer.BaseHTTPRequestHandler.__init__(self,request, client_address, server)
		except:
			pass

    	def do_GET(self):       
		self.__handle("GET")

    	def do_POST(self):      
		self.__handle("POST")
	
	def __handle(self, method):
		try:
			self.do_handle(method)
		except Exception,e:
			import traceback
			traceback.print_exc(file=sys.stdout)

	def send_html(self,code,body):
		response = BaseHTTPServer.BaseHTTPRequestHandler.responses[code][0]
        	self.wfile.write("HTTP/1.0 %s %s\r\n" % (code, response))
        	self.wfile.write("Server: ProtoVer\r\n")
        	self.wfile.write("Content-type: text/html\r\n")
        	self.wfile.write("\r\n")
        	self.wfile.write(body)
		self.wfile.flush()

	def do_handle(self, method):
        	self.path, self.query = urllib.splitquery(self.path)
        	length = self.headers.get("Content-Length")
        	if length:
            		self.content = self.rfile.read(int(length))

		if self.query and len(self.query) > 0:
			self.params = cgi.parse_qs(self.query)
		else:
			self.query = ""
			self.params = {}

		for a in allhandlers:
			handler = a()
			if handler.test(self.path) == 1:
				handler.set_arg("path", self.path)
				handler.set_arg("params", self.params)
				data = handler.run()		
				self.send_html(200, data)
				break

		return 1

def usage():
        print "Usage: %s [options]" % sys.argv[0]
        print "-p port   listen port"
	print "-d level  set debug level (default: 0)"
	sys.exit()


def main():
	print "%s v%s\n"%(DESCRIPTION,VERSION)
	
	if len(sys.argv) < 2:
		usage()

	opts={}
	opts["port"] = 8123
	opts["timeout"] = 3.0
	opts["debug"] = 0	

        try:
                (options, args)=getopt.getopt(sys.argv[1:],"p:w:d")
        except getopt.GetoptError:
                usage()

        for o, a in options:
                if o == "-p":
                        opts["port"] = int(a)
                if o == "-w":
                        opts["timeout"] = float(a)
                if o == "-d":
                        opts["debug"] = int(a)

	print "Loading modules ..."
	ary = utils.load_modules()
	for a in ary[0]:
		alltestmodules.append(a)	
	for a in ary[1]:
		alltestgroups.append(a)
	print "Loaded %d module(s)\n" % len(alltestmodules)

	print "Initalizing request handlers ..."
	allhandlers.append(IndexHandler)
       	allhandlers.append(ModuleHandler)
        allhandlers.append(SessionHandler)
	allhandlers.append(LogHandler)
	print "Loaded %d handler(s)\n" % len(allhandlers)

	print "Running HTTP server on %d port\n" %opts["port"]

	httpd = BaseHTTPServer.HTTPServer(("", opts["port"]), MainRequestHandler)
	httpd.serve_forever()

if __name__=="__main__":
	sys.exit(main())
